#
# spec file for package ghc-semigroupoids
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global pkg_name semigroupoids
%bcond_with tests
Name:           ghc-%{pkg_name}
Version:        5.3.2
Release:        1.3
Summary:        Semigroupoids: Category sans id
License:        BSD-2-Clause
Group:          Development/Libraries/Haskell
URL:            https://hackage.haskell.org/package/%{pkg_name}
Source0:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/%{pkg_name}-%{version}.tar.gz
BuildRequires:  ghc-Cabal-devel
BuildRequires:  ghc-base-orphans-devel
BuildRequires:  ghc-bifunctors-devel
BuildRequires:  ghc-cabal-doctest-devel
BuildRequires:  ghc-comonad-devel
BuildRequires:  ghc-containers-devel
BuildRequires:  ghc-contravariant-devel
BuildRequires:  ghc-distributive-devel
BuildRequires:  ghc-hashable-devel
BuildRequires:  ghc-rpm-macros
BuildRequires:  ghc-tagged-devel
BuildRequires:  ghc-template-haskell-devel
BuildRequires:  ghc-transformers-compat-devel
BuildRequires:  ghc-transformers-devel
BuildRequires:  ghc-unordered-containers-devel
%if %{with tests}
BuildRequires:  ghc-doctest-devel
%endif

%description
Provides a wide array of (semi)groupoids and operations for working with them.

A 'Semigroupoid' is a 'Category' without the requirement of identity arrows for
every object in the category.

A 'Category' is any 'Semigroupoid' for which the Yoneda lemma holds.

When working with comonads you often have the '<*>' portion of an
'Applicative', but not the 'pure'. This was captured in Uustalu and Vene's
"Essence of Dataflow Programming" in the form of the 'ComonadZip' class in the
days before 'Applicative'. Apply provides a weaker invariant, but for the
comonads used for data flow programming (found in the streams package), this
invariant is preserved. Applicative function composition forms a semigroupoid.

Similarly many structures are nearly a comonad, but not quite, for instance
lists provide a reasonable 'extend' operation in the form of 'tails', but do
not always contain a value.

Ideally the following relationships would hold:

> Foldable ----> Traversable <--- Functor ------> Alt ---------> Plus
Semigroupoid > | | | | | > v v v v v > Foldable1 ---> Traversable1 Apply
--------> Applicative -> Alternative Category > | | | | > v v v v > Bind
---------> Monad -------> MonadPlus Arrow >

Apply, Bind, and Extend (not shown) give rise the Static, Kleisli and Cokleisli
semigroupoids respectively.

This lets us remove many of the restrictions from various monad transformers as
in many cases the binding operation or '<*>' operation does not require them.

Finally, to work with these weaker structures it is beneficial to have
containers that can provide stronger guarantees about their contents, so
versions of 'Traversable' and 'Foldable' that can be folded with just a
'Semigroup' are added.

%package devel
Summary:        Haskell %{pkg_name} library development files
Group:          Development/Libraries/Haskell
Requires:       %{name} = %{version}-%{release}
Requires:       ghc-compiler = %{ghc_version}
Requires(post): ghc-compiler = %{ghc_version}
Requires(postun): ghc-compiler = %{ghc_version}

%description devel
This package provides the Haskell %{pkg_name} library development files.

%prep
%setup -q -n %{pkg_name}-%{version}

%build
%ghc_lib_build

%install
%ghc_lib_install

%check
%cabal_test

%post devel
%ghc_pkg_recache

%postun devel
%ghc_pkg_recache

%files -f %{name}.files
%license LICENSE

%files devel -f %{name}-devel.files
%doc CHANGELOG.markdown README.markdown

%changelog
* Sat Jan  5 2019 psimons@suse.com
- Update semigroupoids to version 5.3.2.
  5.3.2 [2019.01.04]
  - -----------------
  * Bump the lower bound on `semigroups` to 0.16.2, and avoid incurring
    the dependency entirely on recent GHCs.
  * Fix the build on GHC 7.0 and 7.2.
* Sat Oct 20 2018 Peter Simons <psimons@suse.com>
- Use https URL to refer to bugs.opensuse.org.
* Wed Jul 18 2018 psimons@suse.com
- Cosmetic: replace tabs with blanks, strip trailing white space,
  and update copyright headers with spec-cleaner.
* Fri Jul 13 2018 psimons@suse.com
- Update semigroupoids to version 5.3.1.
  * Fix a regression introduced in `semigroupoids-5.3` in which some modules
    regressed from `Trustworthy` to `Unsafe`.
  * Allow building with `containers-0.6`.
  * Add `Alt` instances for `First` and `Last` from `Data.Semigroup`, and
    `Alt` and `Plus` instances for `First` and `Last` from `Data.Monoid`.
  * Add missing `Apply`, `Bind`, `Extend`, `Foldable1` and `Traversable1`
    instances for `Data.Semigroups`, `Data.Monoid` and `GHC.Generics`.
* Mon May 14 2018 psimons@suse.com
- Update semigroupoids to version 5.2.2 revision 3.
  * Add `optional` to `Data.Functor.Alt` (analogous to the `optional` function
    in `Control.Applicative`)
  * `liftF2` is now a class method of `Apply` (mirroring the fact that `liftA2`
    is now a class method of `Applicative`). `liftF2` and `(<.>)` have default
    definitions in terms of the other.
  * Allow building with GHC 8.4
  * `Apply` and `Bind` instances for `Q`, from the `template-haskell` package.
  * Add instances for `(:~:)` and `(:~~:)` from `Data.Type.Equality`, and
    `Coercion` from `Data.Type.Coercion`
  * Add the `toNonEmpty` method to `Foldable1`. Add `foldrM1` and `foldlM1`
    functions to `Data.Semigroup.Foldable` that are defined in terms of `toNonEmpty`.
  * Add `Apply`, `Bind`, `Foldable1`, and `Traversable1` instances for `Complex`
  * Add `Apply` and `Bind` instances for `HashMap` from the `unordered-containers` package
    (on which `semigroupoids` now depends)
  * Add `Semigroupoid` instances for `Tagged` and `Const`
* Thu Jul 27 2017 psimons@suse.com
- Update to version 5.2.
* Thu Sep 15 2016 psimons@suse.com
- Update to version 5.1 revision 0 with cabal2obs.
* Sun Jul 10 2016 psimons@suse.com
- Update to version 5.0.1 revision 1 with cabal2obs.
* Mon Jan 18 2016 mimi.vx@gmail.com
- update to 5.0.1
  * transformers-compat 0.5 support
  * Removed some redundant constraints.
* Sun Aug 23 2015 mimi.vx@gmail.com
- update to 5.0.0.4
  * doctest 0.10 support
* Thu Aug 13 2015 mimi.vx@gmail.com
- update to 5.0.0.3
* Thu Aug  6 2015 mimi.vx@gmail.com
- update to 5.0.0.2
  * Added the CHANGELOG to the distribution so that hackage can link to it in the haddocks.
  * Absorbed Data.Bifunctor.Apply, Data.Semigroup.Bifoldable and Data.Semigroup.Traversable
  from bifunctors.
  * This caused us to pick up a dependency on tagged.
  * Exiled Data.Semifunctor.*, Data.Semigroupoid.Product and Data.Semigroupoid.Coproduct
  to semigroupoid-extras.
  * Set an explicit fixity for -<- and ->-.
  * Major changes to the API to support PolyKinds and DataKinds. This necessarily shuts off GHC <= 7.4.
  * Orphan instances have moved upstream into a common base-orphans package.
  * Added asum1 to Data.Semigroup.Foldable.
  * Support for 'ConstrainedClassMethods' is currently required for GHC HEAD.
* Wed May 27 2015 mimi.vx@gmail.com
- correct license to BSD-2-Clause
* Sun Apr 26 2015 mimi.vx@gmail.com
- initial commit
